#!/usr/bin/env python

"""**********************************************************************
  DLMS/COSEM components

  i-cube, 1376 Goumoens-la-Ville, Switzerland
          infos@icube.ch

 (C)Copyright 2004-2012
**********************************************************************

"""

import binascii
import os
import string
import sys
import time
import types
import xml.etree.cElementTree as ET

from icube.memorystream import PduOutputStream, PduInputStream

from icube.utils import Utils
import icube.ezhdlc
import icube.xmlpdu
import icube.wrapper

from icube.xmlpdu.auto.XMLIDs import XmlIDs
from icube.xmlpdu.auto._COSEMpdu import _COSEMpdu
from icube.xmlpdu.infra.xmlstringbuffer import XmlStringBuffer

from icube.wrapper import WRAPPERParameters
from icube.wrapper.wrapperprotocol import WRAPPERProtocol

from comm.socketIo import Client
import icube


class WrapperDemo(object):
    
    def __init__(self, iec=False):
        self._initialize_parameters(iec=iec)
        
    def OnSendStream(self, stream):
        if isinstance(stream, types.ListType):
            print "CB_S:", Utils.bytes_to_hex_string(stream)
        else:
            print "CB_S:", stream
            
    def OnReceivedStream(self, stream):
        if isinstance(stream, types.ListType):
            print "CB_R:", Utils.bytes_to_hex_string(stream)
        else:
            print "CB_R:", stream

    def _initialize_parameters(self, iec=True):
        self._wrapperParameters = WRAPPERParameters()
        self._wrapperParameters.SentStreamCallback = self.OnSendStream
        self._wrapperParameters.ReceivedStreamCallBack = self.OnReceivedStream
        self._wrapperParameters.ClientAddress = 0x10
        self._wrapperParameters.ServerAddress = 1
                
    def process_xml_file(self, file_):
        if True:
            print 6*'-----     '
            print "process_file: %s" % file_
            with open(file_) as xml:
                for line in xml.readlines():
                    print line.rstrip()
                    
            _tree = ET.parse(file_)
            _root = _tree.getroot()
            _pdu = _COSEMpdu(XmlIDs.XMLIgnore, element=_root)
            _pduStream = PduOutputStream()
            _pdu.to_pdu(_pduStream)
            
            _inPdu = self._wrapper.send_receive(_pduStream._buffer)
            
            _inPdu = PduInputStream(_inPdu._buffer)
            _pduFromBytes = _COSEMpdu(XmlIDs.XMLIgnore, pdu=_inPdu)
            _xml = XmlStringBuffer()
            _pduFromBytes.to_xml(_xml)
            _xmlStr = _xml.buffer
            print _xmlStr
            
#        except:
#            cla, exc, trbk =  sys.exc_info()
#            print exc

    def doIt(self, files):
        print "WrapperDemo"
        print icube.ezhdlc.get_version_str()
        print icube.xmlpdu.get_version_str()
        print icube.wrapper.get_version_str()
        print
        
        IOStream = Client(host='127.0.0.1', port=2048)
        IOStream.open()
        IOStream.setTimeout(3000)

        self._wrapper = WRAPPERProtocol(self._wrapperParameters, IOStream=IOStream)
        
        try:
            for _file in files:
                self.process_xml_file(_file)
        finally:
            IOStream.close()
        
if __name__ == '__main__':
    print sys.argv

    def usage():
        """
    usage: 
        python wrapperdemo.py file(s)
            connect to server, process file(s) 
                xml->pdu
                pdu->server->inpdu
                inpdu->xml 
    """
        print usage.__doc__

#    print os.getcwd(), len(sys.argv), sys.argv
    if len(sys.argv) > 1:
        WrapperDemo().doIt(sys.argv[1:])
    else:
        usage()

