#!/usr/bin/env python

"""**********************************************************************
  DLMS/COSEM components

  i-cube, 1376 Goumoens-la-Ville, Switzerland
          infos@icube.ch

 (C)Copyright 2004-2012
**********************************************************************

"""

import os
import string
import sys
import xml.etree.cElementTree as ET

from icube.memorystream import PduOutputStream, PduInputStream

from icube.utils import Utils
from icube.xmlpdu.auto.XMLIDs import XmlIDs
from icube.xmlpdu.auto._COSEMpdu import _COSEMpdu
from icube.xmlpdu.infra.xmlstringbuffer import XmlStringBuffer


class XmlPduTest(object):
    '''
    classdocs
    '''

    def process_file(self, file_):
        '''
        '''
        _root, _ext = os.path.splitext(file_)
        if string.lower(_ext) != '.xml':
            return
        #print 80*'-'
        print "process_file: %s" % file_
        
        _dir, _file = os.path.split(file_.rstrip('/'))
        _dir, _s1, _s2  = _dir.rpartition('/') 
        _dir, _s1, _s2  = _dir.rpartition('/') 
        _pdu_file = _file[:len(_file)-4]             # remove ".xml"

        _tree = ET.parse(file_)
        _root = _tree.getroot()
        _pdu = _COSEMpdu(XmlIDs.XMLIgnore, element=_root)
        
        _pduStream = PduOutputStream()
        _pdu.to_pdu(_pduStream)
        _pduStr = Utils.bytes_to_hex_string(_pduStream._buffer)
        with open(_dir + '/py/pdu/' + _pdu_file + '.txt', 'w+') as _out:
            _out.write(_pduStr)
        
        _pduBytes = PduInputStream(_pduStream._buffer)
        _pduFromBytes = _COSEMpdu(XmlIDs.XMLIgnore, pdu=_pduBytes)
        
        #print 40*"-"
        #print
        _xml = XmlStringBuffer()
        _pduFromBytes.to_xml(_xml)
        _xmlStr = _xml.buffer
        _xmlStr = _xmlStr.rstrip('\n')
        
        with open(_dir + '/py/xml/' + _file, 'w+') as _out:
            _out.write(_xmlStr)
        
        #print
            
    def process_directory(self, dirname):
        errors = False
        for _root, _dirs, files in os.walk(dirname):
            for _file in files:
                _filename = os.path.join(_root, _file)
                try:
                    self.process_file(_filename)
                except Exception, e:
                    errors = True
                    print '  ', 12*'*', e,  12*'*'
        if errors:
            print 12*'*',"error(s) detected",  12*'*'
            
    def doIt(self, argv):
        #print argv
        _path = argv[0]
        if os.path.isdir(_path):
            #print "directory: %s" % _path
            self.process_directory(_path)
        else:
            #print "file: %s" % _path
            try:
                self.process_file(_path)
            except Exception, e:
                print '  ', 12*'*', e,  12*'*'
            
if __name__ == '__main__':
    print sys.argv

    def usage():
        """
    usage: 
        python xmlpdutest.py file | dir
            file or all files in dir are converted from xml and back
            the resulting file(s)
    """
        print usage.__doc__

    print os.getcwd(), len(sys.argv), sys.argv
    if len(sys.argv) > 1:
        XmlPduTest().doIt(sys.argv[1:])
    else:
        usage()

